@@ -2,13 +2,18 @@ from __future__ import annotations
 
 from typing import ContextManager
 
+import einspect._patch
 from einspect.type_orig import orig
+from einspect.types import NULL, ptr
 from einspect.views.factory import view
 from einspect.views.unsafe import global_unsafe
 from einspect.views.view_type import impl
 
-__all__ = ("view", "unsafe", "impl", "orig")
+# Runtime patches
+einspect._patch.run()
 
-__version__ = "0.4.9"
+__all__ = ("view", "unsafe", "impl", "orig", "ptr", "NULL")
+
+__version__ = "0.5.1"
 
 unsafe: ContextManager[None] = global_unsafe
