@@ -5,7 +5,12 @@ from einspect.structs.py_object import PyObject, PyVarObject  # Req import first
 from einspect.structs.py_type import PyTypeObject
 from einspect.structs.py_long import PyLongObject  # Req before PyBoolObject
 from einspect.structs.py_bool import PyBoolObject
-from einspect.structs.py_unicode import PyUnicodeObject
+from einspect.structs.py_float import PyFloatObject
+from einspect.structs.py_unicode import (
+    PyUnicodeObject,
+    PyCompactUnicodeObject,
+    PyASCIIObject,
+)
 from einspect.structs.py_list import PyListObject
 from einspect.structs.py_tuple import PyTupleObject
 from einspect.structs.py_set import PySetObject
@@ -21,7 +26,10 @@ __all__ = (
     "PyListObject",
     "PyLongObject",
     "PyBoolObject",
+    "PyFloatObject",
     "PyUnicodeObject",
+    "PyCompactUnicodeObject",
+    "PyASCIIObject",
     "PyDictObject",
     "MappingProxyObject",
     "PySetObject",
