@@ -2,17 +2,31 @@
 from __future__ import annotations
 
 import ctypes
-from ctypes import Structure, c_void_p, pythonapi
-from typing import TYPE_CHECKING, Dict, Generic, List, Tuple, Type, TypeVar, Union
+import warnings
+from contextlib import suppress
+from ctypes import POINTER, Structure, c_void_p, pythonapi
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Dict,
+    Generic,
+    List,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+    overload,
+)
 
 from typing_extensions import Annotated, Self
 
+from einspect.api import PTR_SIZE, address, align_size
 from einspect.compat import Version, python_req
 from einspect.protocols.delayed_bind import bind_api
 from einspect.protocols.type_parse import is_ctypes_type
 from einspect.structs.deco import struct
 from einspect.structs.py_gc import PyGC_Head
-from einspect.structs.traits import AsRef
+from einspect.structs.traits import AsRef, IsGC
 from einspect.types import ptr
 
 if TYPE_CHECKING:
@@ -25,6 +39,8 @@ _KT = TypeVar("_KT")
 _VT = TypeVar("_VT")
 _ST = TypeVar("_ST", bound=Structure)
 
+DEFAULT = object()
+
 
 @struct
 class PyObject(Structure, AsRef, Generic[_T, _KT, _VT]):
@@ -32,8 +48,59 @@ class PyObject(Structure, AsRef, Generic[_T, _KT, _VT]):
 
     ob_refcnt: int
     ob_type: Annotated[ptr[PyTypeObject[Type[_T]]], c_void_p]
+
     _fields_: List[Union[Tuple[str, type], Tuple[str, type, int]]]
-    _from_type_name_: str
+
+    @overload
+    def __new__(cls, __obj: _T):
+        ...
+
+    @overload
+    def __new__(cls, *, ob_refcnt: int = 1, ob_type: ptr[PyTypeObject], **kwargs):
+        ...
+
+    def __new__(cls, __obj: _T = DEFAULT, **kwargs):
+        """
+        Create a new PyObject.
+
+        One positional argument can be passed to create a PyObject from an object.
+        Otherwise, keyword arguments of fields can be provided.
+
+        Args:
+            __obj: A Python object to create a PyObject from.
+            **kwargs: Fields to create a PyObject from.
+        """
+        from einspect.structs.py_type import PyTypeObject
+
+        # Base case
+        if __obj is DEFAULT and not kwargs:
+            return super().__new__(cls)
+        if __obj is not DEFAULT:
+            # If already a PyObject
+            if isinstance(__obj, PyObject):
+                return cls.from_address(__obj.address)
+            # Use from_object
+            return cls.from_object(__obj)
+        # Check kwargs
+        if "ob_type" not in kwargs:
+            raise TypeError("Missing required keyword-argument field 'ob_type'")
+
+        ob_size: int = kwargs.get("ob_size", 0).__index__()
+        ob_type = PyTypeObject.try_from(kwargs["ob_type"])
+
+        if issubclass(cls, IsGC):
+            res = (
+                ob_type.GC_NewVar(ob_size)
+                if issubclass(cls, PyVarObject)
+                else ob_type.GC_New()
+            )
+        else:
+            res = (
+                ob_type.NewVar(ob_size)
+                if issubclass(cls, PyVarObject)
+                else ob_type.New()
+            )
+        return res.contents.astype(cls)
 
     @property
     def mem_size(self) -> int:
@@ -45,17 +112,6 @@ class PyObject(Structure, AsRef, Generic[_T, _KT, _VT]):
         """Return the address of the PyObject."""
         return ctypes.addressof(self)
 
-    @property
-    def _orig_type_name(self) -> str | None:
-        """
-        Return the type repr of the original object.
-
-        Only available if instance created with from_object.
-        """
-        if not hasattr(self, "_from_type_name_"):
-            return None
-        return self._from_type_name_
-
     def __eq__(self, other: Self) -> bool:
         """Return True if the PyObject is equal in address to the other."""
         if not isinstance(other, PyObject):
@@ -65,9 +121,11 @@ class PyObject(Structure, AsRef, Generic[_T, _KT, _VT]):
     def __repr__(self) -> str:
         """Return a string representation of the PyObject."""
         cls_name = f"{self.__class__.__name__}"
-        type_name = self._orig_type_name
-        if type_name:
-            cls_name += f"[{type_name}]"
+        # For generic PyObjects, add the type name
+        if self.__class__ in (PyObject, PyVarObject):
+            with suppress(ValueError):
+                obj_type = self.ob_type.contents
+                cls_name += f"[{obj_type.tp_name.decode()}]"
         return f"<{cls_name} at {self.address:#04x}>"
 
     def _format_fields_(self) -> Fields:
@@ -82,15 +140,7 @@ class PyObject(Structure, AsRef, Generic[_T, _KT, _VT]):
     @classmethod
     def from_object(cls, obj: _T) -> Self:
         """Create a PyObject from an object."""
-        # Record the type name for later repr use
-        type_repr = str(type(obj).__name__)
-        py_obj = ctypes.py_object(obj)
-        addr = ctypes.c_void_p.from_buffer(py_obj).value
-        if addr is None:
-            raise ValueError("Object is not a valid pointer")
-        inst = cls.from_address(addr)
-        inst._from_type_name_ = type_repr
-        return inst
+        return cls.from_address(address(obj))
 
     @classmethod
     def from_gc(cls, gc: PyGC_Head) -> Self:
@@ -163,6 +213,37 @@ class PyObject(Structure, AsRef, Generic[_T, _KT, _VT]):
             return self.gc_is_tracked()
         return True
 
+    def instance_dict(self) -> ptr[PyObject[dict, Any, Any]] | None:
+        """
+        Return the instance dict of the PyObject.
+
+        An offset override can be set by `__st_dictoffset__`. It should be
+        relative to the address of the PyObject.
+
+        https://docs.python.org/3/c-api/typeobj.html#c.PyTypeObject.tp_dictoffset
+        """
+        # Get the tp_dictoffset of the type
+        offset = self.ob_type.contents.tp_dictoffset
+        # If 0, the type does not have a dict
+        if offset == 0:
+            return None
+        # For > 0, start from the address of the PyObject
+        if offset > 0:
+            addr = self.address + offset
+        # For < 0, start after the struct
+        else:
+            # If not a PyVarObject, use __basic_size__ instead
+            if getattr(self, "ob_size", None) is None:
+                size = self.ob_type.contents.tp_basicsize
+            else:
+                size = align_size(self.mem_size, PTR_SIZE)
+                # Increase size by pointer size since mem_size at this point
+                # excludes the instance dict (unlike __sizeof__)
+                size += PTR_SIZE
+            addr = self.address + size + offset
+        # Return the pointer
+        return POINTER(PyObject).from_address(addr)
+
     @bind_api(python_req(Version.PY_3_10) or pythonapi["Py_NewRef"])
     def NewRef(self) -> object:
         """Returns new reference of the PyObject."""
@@ -184,6 +265,23 @@ class PyObject(Structure, AsRef, Generic[_T, _KT, _VT]):
         """Set the attribute of the PyObject."""
 
 
+# We don't want this visible to type-checking but `PyObject.__init__`
+# needs to ignore the `__obj` positional argument
+orig_init = PyObject.__init__  # type: ignore
+
+
+def _PyObject__init__(self, *args, **kwargs) -> None:
+    if len(args) > 1:
+        warnings.warn(
+            "Positional arguments to PyObject.__init__ are not supported, use keywords instead.",
+            UserWarning,
+        )
+    return orig_init(self, **kwargs)  # type: ignore
+
+
+PyObject.__init__ = _PyObject__init__
+
+
 @struct
 class PyVarObject(PyObject[_T, _KT, _VT]):
     """
