@@ -21,17 +21,44 @@ __all__ = ("StrView",)
 _T = TypeVar("_T")
 
 
+def _check_resize(v: StrView, target: int, method: str) -> None:
+    """
+    Check if the string can be resized to `target` size.
+
+    Raises:
+        UnsafeError: If the string cannot be resized, and not in an unsafe context.
+    """
+    # See if resizing is safe
+    if target > v.mem_allocated and not v._unsafe:
+        raise UnsafeError(
+            f"string {method} required {target} bytes, beyond current"
+            f" allocated memory of {v.mem_allocated} bytes."
+            " Enter an unsafe context to allow this."
+        )
+
+
+def _str_move(v: StrView, target: str) -> None:
+    """Move the string to a new location."""
+    # Since the struct type might change, use a memmove
+    target_v = StrView(target, ref=False)
+    with target_v.unsafe():
+        target_v.move_to(v)
+    v._narrow_type()
+    # On modifications, unintern the string
+    v._pyobject.interned = 0
+
+
 class StrView(View[str, None, None], MutableSequence):
     _pyobject: Union[PyASCIIObject, PyCompactUnicodeObject, PyUnicodeObject]
 
     def __init__(self, obj: str, ref: bool = REF_DEFAULT) -> None:
-        """View a Python string object."""
+        """View a string object."""
         super().__init__(obj, ref)
         # Start with PyASCIIObject, check if we can use a more specific type
         self._narrow_type()
 
     def _narrow_type(self) -> None:
-        # Narrow to a more specific unicode type if possible
+        """Narrow to a more specific unicode type if possible."""
         if self._pyobject.compact:
             if self._pyobject.ascii:
                 self._pyobject = self._pyobject.astype(PyASCIIObject)
@@ -63,27 +90,21 @@ class StrView(View[str, None, None], MutableSequence):
         ...
 
     def __setitem__(self, index: int | slice, value: str | Iterable[str]) -> None:
+        if not isinstance(index, slice):
+            index = index.__index__()
+            if len(value) > 1:
+                raise ValueError(
+                    "cannot set integer indices with string of length > 1, use a slice instead."
+                )
         # Use a temp list for the slice calculation
         temp = list(self._pyobject.into_object())
         temp[index] = value
         # Combine to str
         target = "".join(temp)
 
-        # See if resizing is safe
-        if target.__sizeof__() > self.mem_allocated and not self._unsafe:
-            raise UnsafeError(
-                "setitem required str to be resized beyond current memory allocation."
-                " Enter an unsafe context to allow this."
-            )
-
-        # Since the struct type might change, use a memmove
-        v = StrView(target, ref=False)
-        with self.unsafe(), v.unsafe():
-            v.move_to(self)
-        self._narrow_type()
-
-        # On modifications, unintern the string
-        self._pyobject.interned = 0
+        # See if resizing is safe, then move the string
+        _check_resize(self, target.__sizeof__(), "setitem")
+        _str_move(self, target)
 
     @overload
     def __delitem__(self, index: int) -> None:
@@ -99,22 +120,9 @@ class StrView(View[str, None, None], MutableSequence):
         del temp[index]
         # Combine to str
         target = "".join(temp)
-
-        # See if resizing is safe
-        if target.__sizeof__() > self.mem_allocated and not self._unsafe:
-            raise UnsafeError(
-                "setitem required str to be resized beyond current memory allocation."
-                " Enter an unsafe context to allow this."
-            )
-
-        # Since the struct type might change, use a memmove
-        v = StrView(target, ref=False)
-        with self.unsafe(), v.unsafe():
-            v.move_to(self)
-        self._narrow_type()
-
-        # On modifications, unintern the string
-        self._pyobject.interned = 0
+        # See if resizing is safe, then move the string
+        _check_resize(self, target.__sizeof__(), "delitem")
+        _str_move(self, target)
 
     def insert(self, index: SupportsIndex, value: str) -> None:
         """Insert object before index."""
@@ -124,22 +132,21 @@ class StrView(View[str, None, None], MutableSequence):
         temp.insert(index, value)
         # Combine to str
         target = "".join(temp)
+        # See if resizing is safe, then move the string
+        _check_resize(self, target.__sizeof__(), "insert")
+        _str_move(self, target)
 
-        # See if resizing is safe
-        if target.__sizeof__() > self.mem_allocated and not self._unsafe:
-            raise UnsafeError(
-                "insert required str to be resized beyond current memory allocation."
-                " Enter an unsafe context to allow this."
-            )
+    def remove(self, value: str) -> None:
+        """
+        Remove first occurrence of substring value.
 
-        # Since the struct type might change, use a memmove
-        v = StrView(target, ref=False)
-        with self.unsafe(), v.unsafe():
-            v.move_to(self)
-        self._narrow_type()
-
-        # On modifications, unintern the string
-        self._pyobject.interned = 0
+        Raises ValueError if the value is not present.
+        """
+        # Equivalent to str.replace(value, "", 1)
+        target = str.replace(self._pyobject.into_object(), value, "", 1)
+        # See if resizing is safe (should always be safe but check anyway)
+        _check_resize(self, target.__sizeof__(), "remove")
+        _str_move(self, target)
 
     @property
     def length(self) -> int:
